
#include "decs.h"

/* diagnostics subroutine */

/*

	call codes:
	0: initial
	1: logfile call
	2: dumpfile call

*/

void diag(int call_code)
{
	char dfnam[10], ifnam[100];
	double ek, eki, ekx, eky, eth, ethi, etot, px, pxi, py, pyi,
	    mass, Qi, Q, Ti, Tav, taui, tau_avg, lam, lami, tci, tc_avg,
	    tcav, Pratioi, Pratio, min_tci, min_tc, e_fin, eg, egi, m_fin,
	    vxa, vya;
	FILE *dump_file, *im_file;
	int i, j;
	void dump(FILE * fp);
	void restart_dump();

	static double e_init, m_init, tdump, timage, area;
	static FILE *ener_file;

	if (call_code == 0) {
		ener_file = fopen("ener.out", "a");
		if (ener_file == NULL) {
			fprintf(stderr,
				"error opening energy output file\n");
			exit(1);
		}
		tdump = t - 1.e-12;
		timage = t - 1.e-12;
		area = Lx * Ly;
		//period = 2.*M_PI/omega ;
		if (is_restart != 1)
			fprintf(ener_file,
				"#config: pamp = %.5g, mu = %.5g, T0 = %.5g, rho0 = %.5g, tc0 = %.5g, tau0 = %.5g, ProPg0 = %.5g, tcmin = %g, cool_fac = %g, Q = %g, G = %g, Lx = %g, Ly = %g, NX = %d, NY = %d\n",
				pamp, mu, T0, rho0cgs, tc0, tau0, ProPg0,
				tcmin, cool_fac, Qeq, G, Lx, Ly, NX, NY);
	}

	/* calculate energies */
	ek = 0.;
	eth = 0.;
	eg = 0.;
	etot = 0.;
	px = py = 0.;
	mass = Q = tau_avg = tc_avg = lam = Tav = Pratio = min_tc = 0.;
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vxa = 0.5 * (vx[i + 1][j] + vx[i][j]);
			vya = 0.5 * (vy[i][j + 1] + vy[i][j]);
			pxi = vxa * S[i][j];
			pyi = vya * S[i][j];

			ekx = 0.5 * S[i][j] * vxa * vxa;
			eky = 0.5 * S[i][j] * vya * vya;
			eki = ekx + eky;

			egi = 0.5 * S[i][j] * pot[i][j];

			if (fabs(g2 - 1.) > 1.e-6)
				ethi = e[i][j];
			else
				ethi = cs * cs * S[i][j];

			Qi = sqrt(g2 * (g2 - 1) * e[i][j]) / (M_PI *
							      S[i][j] *
							      sqrt(S[i]
								   [j]));
			taui = tau[i][j];
			tci = tcnew[i][j];
			Ti = Tm[i][j];
			lami = e[i][j] / tcnew[i][j];
			Pratioi = ProPg[i][j];
			min_tci = mintc[i][j];

			ek += eki * dx * dy;
			eth += ethi * dx * dy;
			eg += egi * dx * dy;
			px += pxi * dx * dy;
			py += pyi * dx * dy;
			mass += S[i][j] * dx * dy;
			Q += Qi * dx * dy / area;
			tau_avg += taui * dx * dy / area;
			tc_avg += tci * dx * dy / area;
			lam += lami * dx * dy;
			Tav += Ti * dx * dy / area;
			Pratio += Pratioi * dx * dy / area;
			min_tc += min_tci * dx * dy / area;
		}
	etot = ek + eth + eg;
	tcav = eth / lam;

	/* drop out condition */
	if (eg / area < -300.) {
		fprintf(stdout, "model has collapsed, t = %.5g, eg = %g\n",
			t, eg / area);
		exit(0);
	}

	/* calculate energies */
	if (call_code == 0) {
		e_init = etot;
		m_init = 0.;
		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++)
				m_init += S[i][j] * dx * dy;
	} else if (call_code == 2) {
		e_fin = etot;
		m_fin = 0.;
		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++)
				m_fin += S[i][j] * dx * dy;
		fprintf(stderr, "\n\nEnergy: ini,fin,del: %g %g %g\n",
			e_init, e_fin, (e_fin - e_init) / (fabs(e_init) +
							   SMALL));
		fprintf(stderr, "mass: ini,fin,del: %g %g %g\n",
			m_init, m_fin, (m_fin - m_init) / (fabs(m_init) +
							   SMALL));
	} else {

		fprintf(ener_file, "%10.5g\t%10.5g\t%10.5g\t%10.5g\t", t,
			ek / area, eth / area, eg / area);
		fprintf(ener_file, "%10.5g\t%10.5g\t%10.5g\t%10.5g\t",
			ekx / area, eky / area, px / area, py / area);
		fprintf(ener_file,
			"%10.5g\t%10.5g\t%10.5g\t%10.5g\t%10.5g\t%10.5g\t%10.5g\t%10.5g\n",
			alphag, alphah, Q, tau_avg, tc_avg, tcav, Tav,
			Pratio);
		fflush(ener_file);
	}

	/* dump at regular intervals */
	if (t >= tdump || call_code == 2) {
		fprintf(stderr, "t: %lf %lf %lf\n", t, tdump, DTd);
		sprintf(dfnam, "dump%03d", dump_cnt);
		dump_file = fopen(dfnam, "w");

		if (dump_file == NULL) {
			fprintf(stderr, "error opening dump file\n");
			exit(2);
		}

		dump(dump_file);
		fclose(dump_file);

		dump_cnt++;
		tdump += DTd;

		/*
		   restart_dump() ;
		 */
	}

	/* make image of density at regular intervals */
	if (t >= timage || call_code == 2) {

		/* possible to repeat this for any variables desired */
		sprintf(ifnam, "imrho%04d.ppm", im_cnt);
		im_file = fopen(ifnam, "w");

		if (im_file == NULL) {
			fprintf(stderr, "error opening image file\n");
			exit(2);
		}

		image_ppm(im_file, S);
		fclose(im_file);

		im_cnt++;
		timage += DTi;
	}
}
