
#include "decs.h"

void init()
{
	int i, j;

	void set_arrays();
	void zero_arrays();
	void perturb(double amp);
	void scaling();
	int restart();

	double X, Y, Xp, Yp, ampl;

	g2 = 11. / 7.;
	q = 1.5;
	W = 1.;
	S0 = 1.;
	G = 1.;
	Qeq = 1.;
	cs = cs0 = M_PI * Qeq;
	ampl = 0.0001;

#if FULLCOOL
	/* set scaling between cgs units and simulation units */
	scaling();
#endif

	tcmin = 0.0 * tc0;
	tcnew_min = 1. / SMALL;

	if (g2 > 1.)
		u0 = S0 * cs * cs / (g2 * (g2 - 1.));
	else
		u0 = cs * cs;

	fprintf(stderr, "Lx,Ly,NX,NY,g2: %lf %lf %d %d %lf\n", Lx, Ly, NX,
		NY, g2);

	dx = Lx / (double) NX;
	dy = Ly / (double) NY;
	cour = 0.45;

	nu_vnr = 2.0;
	nu_l = 0.;

	dump_cnt = 0;
	im_cnt = 0;

	/* set up arrays,details */
	set_arrays();
	zero_arrays();

	dt = 1.e-4 / W;

	if (is_restart == 1)
		restart();
	else {
		t = 0.;
		//dt = 1.e-4/W ;  /* needs initial value */
		DTd = tf / 20.;	/* dumping frequency */
		DTi = tf / 800.;	/* image frequency */
		DTl = DTd / 1000.;	/* log frequency */

		dump_cnt = 0;
		im_cnt = 0;

		kx0 = -2. * 2. * M_PI / Lx;
		ky0 = 2. * M_PI / Ly;

		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++) {
				X = (i + .5) * dx - 0.5 * Lx;
				Xp = i * dx - 0.5 * Lx;
				Y = (j + .5) * dy - 0.5 * Ly;
				Yp = j * dy - 0.5 * Ly;

				vx[i][j] =
				    0. * ampl * cos(kx0 * Xp + ky0 * Y);
				vy[i][j] =
				    0. * ampl * cos(kx0 * X + ky0 * Yp);
				S[i][j] =
				    S0 + 0. * ampl * cos(kx0 * X +
							 ky0 * Y);
				e[i][j] =
				    u0 + 0. * ampl * cos(kx0 * X +
							 ky0 * Y);
				tau[i][j] = tau0;
				tcnew[i][j] = tc0;
				Tm[i][j] = T0;
				rgm[i][j] = regime;
				ProPg[i][j] = ProPg0;
				flr[i][j] = S0;
				pot[i][j] = 0.;
			}

		/* add in white noise to initial velocities */
		perturb(pamp);
	}

	/* enforce boundary conditions */
	bound_var(S, t);
	bound_var(e, t);
	bound_var(vx, t);
	bound_var(vy, t);

}
