
/* generate opacity table from power-law fits */

#include "decs.h"

REAL opac(REAL rho, REAL T)
{
	REAL lop, lrho, lT;

	if (rho <= 0. || T <= 0.) {
		fprintf(stdout, "error: rho or T negative: %g, %g\n", rho,
			T);
		exit(1);
	}

	/* do everything in terms of log of density, temperature,
	   and opacity.  Log(opacity) = lop */
	lrho = log(rho);
	lT = log(T);

	if (lT < log(166.810)) {
		/* ice grains */
		lop = -8.517193191 + 2. * lT;
		regime = '1';
		mu = 2.4;
	} else if (lT < log(202.67678165)) {
		/* ice sublimation */
		lop = 37.53450867 - 7. * lT;
		regime = '2';
		mu = 2.4;
	} else if (lT < log(2286.78) + (2. / 49.) * lrho) {
		/* metal grain */
		lop = -2.302585093 + 0.5 * lT;
		regime = '3';
		mu = 2.4;
	} else if (lT < log(2029.75) + (1. / 81.) * lrho) {
		/* metal grain evap */
		lop = 187.2025397 + lrho - 24. * lT;
		regime = '4';
		mu = 2.4;
	} else if (lT < log(10000.) + (1. / 21.) * lrho) {
		/* molecules */
		lop = -18.42068074 + (2. / 3.) * lrho + 3. * lT;
		regime = '5';
		mu = 0.615;
	} else if (lT < log(31195.) + (4. / 75.) * lrho) {
		/* H-scattering */
		lop = -82.89306335 + (1. / 3.) * lrho + 10. * lT;
		regime = '6';
		mu = 0.615;
	} else if (lT < log(1.79395e8) + (2. / 5.) * lrho) {
		/* bound-free, free-free */
		lop = 46.45716697 + lrho - (5. / 2.) * lT;
		regime = '7';
		mu = 0.615;
	} else {
		/* electron scattering */
		lop = -1.055552799;
		regime = '8';
		mu = 0.615;
	}

	/* convert from log(opacity) to opacity & return */
	return (exp(lop));
}
