
#include "decs.h"

#define KCRIT	(Lx/(2.*M_PI))/(M_PI*Qeq*Qeq)
#define RANGE   4.

void perturb(double amp)
{
	void gauss_gen(REAL(*az)[NY + 4], double n);
	REAL(*az)[NY + 4];
	double norm;
	int i, j;

	az = work1;

	/* generate random fields */
	fprintf(stderr, "1\n");
	gauss_gen(az, 0.0);
	fprintf(stderr, "2\n");
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vx[i][j] = az[i][j];
		}

	fprintf(stderr, "3\n");
	gauss_gen(az, 0.0);
	fprintf(stderr, "4\n");
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vy[i][j] = az[i][j];
		}

	/* normalize so that rms velocity is amp */
	norm = 0.;
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			norm += (vx[i][j] * vx[i][j] +
				 vy[i][j] * vy[i][j]) / (NX * NY);
		}
	norm = amp / sqrt(norm);
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vx[i][j] *= norm;
			vy[i][j] *= norm;
		}
	fprintf(stderr, "5\n");

}

/* 

   add Gaussian random field to matrix f, with power spectrum
   amplitude norm 
   
*/

#include <fftw3.h>

void gauss_gen(double (*f)[NY + 4], double sl)
{
	double phase, amp;
	void powsp(double k2, double sl, double *amp, double *phase);
	int i, j, k, ip, jp;

	static fftw_complex *dk, *d;
	static fftw_plan p;
	static int firstc = 1;
	static int ncalls = 0;

	if (firstc) {
		firstc = 0;
		dk = (fftw_complex *) malloc(NX * NY *
					     sizeof(fftw_complex));
		d = (fftw_complex *) malloc(NX * NY *
					    sizeof(fftw_complex));
		p = fftw_plan_dft_2d(NX, NY, dk, d, FFTW_BACKWARD,
				     FFTW_MEASURE);
	}

	if (ncalls % 2 == 0) {

		k = 0;
		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++) {
				k++;
				dk[k][0] = dk[k][1] = 0.;
			}

		/* generate complex random field; use only real part */
		k = 0;
		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++) {
				ip = (i + NX / 2) % NX - NX / 2;
				jp = (j + NY / 2) % NY - NY / 2;

				if (ip * ip + jp * jp >
				    (KCRIT / RANGE) * (KCRIT / RANGE)
				    && ip * ip + jp * jp <
				    (KCRIT * RANGE) * (KCRIT * RANGE)) {
					powsp((double) (ip * ip + jp * jp),
					      sl, &amp, &phase);
				} else {
					phase = 0.;
					amp = 0.;
				}

				dk[k][0] = (REAL) amp *cos(phase);
				dk[k][1] = (REAL) amp *sin(phase);
				k++;
			}

		/* fft */
		fftw_execute(p);
	}

	k = 0;
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			f[i][j] = d[k][ncalls % 2];
			k++;
		}

	return;
}

#include <gsl/gsl_rng.h>

void powsp(double k2, double sl, double *amp, double *phase)
{
	double x1, x2;

	static int firstc = 1;
	static gsl_rng *r;

	if (firstc) {
		firstc = 0;
		/* Mersenne twister */
		r = gsl_rng_alloc(gsl_rng_mt19937);
		gsl_rng_set(r, 1);	/* always use same seed */
	}

	*amp = pow(k2, 0.5 * sl);

	x1 = gsl_rng_uniform(r);
	x2 = gsl_rng_uniform(r);
	*amp *= sqrt(-2. * log(x1)) * cos(2. * M_PI * x2);
	*phase = 2. * M_PI * gsl_rng_uniform(r);

	return;
}
