
#include "decs.h"

void sweepy()
{
	double vyp, mdotp;
	REAL(*dq)[NY + 4];
	REAL(*mdot)[NY + 4];
	REAL(*px)[NY + 4], (*py)[NY + 4];
	void bound_var(REAL(*var)[NY + 4], double t);
	double slope_lim(double y1, double y2, double y3);
	int i, j;

	px = work1;
	py = work2;
	mdot = work3;
	dq = work5;

	/* transform to momenta */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			px[i][j] =
			    0.5 * (S[i - 1][j] + S[i][j]) * vx[i][j];
			py[i][j] =
			    0.5 * (S[i][j - 1] + S[i][j]) * vy[i][j];
		}
	/* no need to bound px,py */

	/* create dimensionless transport variable */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			vy[i][j] *= dt / dy;
	bound_var(vy, t);

	/* calculate mass flux */
	for (i = 0; i < NX; i++)
		for (j = -1; j < NY; j++)
			dq[i][j] =
			    slope_lim(S[i][j - 1], S[i][j], S[i][j + 1]);

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			mdot[i][j] = (vy[i][j] > 0.) ?
			    vy[i][j] * (S[i][j - 1] +
					(1. - vy[i][j]) * dq[i][j -
								1]) :
			    vy[i][j] * (S[i][j] -
					(1. + vy[i][j]) * dq[i][j]);
		}
	bound_var(mdot, t);
	/* wait to update density */

	/* get energy flux */
	for (i = 0; i < NX; i++)
		for (j = -1; j < NY + 1; j++)
			dq[i][j] =
			    slope_lim(e[i][j - 1] / S[i][j - 1],
				      e[i][j] / S[i][j],
				      e[i][j + 1] / S[i][j + 1]);

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY + 1; j++) {
			fl[i][j] = (vy[i][j] > 0) ?
			    mdot[i][j] * (e[i][j - 1] / S[i][j - 1] +
					  (1. - vy[i][j]) * dq[i][j -
								  1]) :
			    mdot[i][j] * (e[i][j] / S[i][j] -
					  (1. + vy[i][j]) * dq[i][j]);
		}

	/* step energy */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			e[i][j] += -(fl[i][j + 1] - fl[i][j]);
			if (e[i][j] < 0.)
				fprintf(stdout,
					"sweepy e < 0 %d %d %g %g\n", i, j,
					S[i][j], e[i][j]);
		}
	bound_var(e, t);

	/* step mass */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			S[i][j] += -(mdot[i][j + 1] - mdot[i][j]);
		}
	bound_var(S, t);

	/* get vx flux */
	for (i = 0; i < NX; i++)
		for (j = -1; j < NY + 1; j++)
			dq[i][j] =
			    slope_lim(vx[i][j - 1], vx[i][j],
				      vx[i][j + 1]);

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY + 1; j++) {
			vyp = 0.5 * (vy[i][j] + vy[i - 1][j]);
			mdotp = 0.5 * (mdot[i][j] + mdot[i - 1][j]);
			fl[i][j] = (vyp > 0) ?
			    mdotp * (vx[i][j - 1] +
				     (1. - vyp) * dq[i][j -
							1]) : mdotp *
			    (vx[i][j] - (1. + vyp) * dq[i][j]);
		}

	/* step vx */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			px[i][j] += -(fl[i][j + 1] - fl[i][j]);

	/* get vy flux */
	for (i = 0; i < NX; i++)
		for (j = -1; j < NY + 1; j++)
			dq[i][j] =
			    slope_lim(vy[i][j - 1], vy[i][j],
				      vy[i][j + 1]);

	for (i = 0; i < NX; i++)
		for (j = -1; j < NY; j++) {
			vyp = 0.5 * (vy[i][j] + vy[i][j + 1]);
			mdotp = 0.5 * (mdot[i][j] + mdot[i][j + 1]);
			fl[i][j] = (vyp > 0.) ?
			    mdotp * (vy[i][j] * (dy / dt) +
				     (1. -
				      vyp) * dq[i][j]) : mdotp * (vy[i][j +
									1]
								  * (dy /
								     dt) -
								  (1. +
								   vyp) *
								  dq[i][j +
									1]);
		}

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			py[i][j] += -(fl[i][j] - fl[i][j - 1]);

	/* return to velocities */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vx[i][j] = 2. * px[i][j] / (S[i][j] + S[i - 1][j]);
			vy[i][j] = 2. * py[i][j] / (S[i][j - 1] + S[i][j]);
		}

	bound_var(vx, t);
	bound_var(vy, t);
}
