
#include "decs.h"

void timestep()
{
	int i, j;
	double idt1, idt2;
	REAL(*dtinv)[NY + 4];
	double dtinv_max;
	static double dtlast;
	void dump(FILE * fp);

	dtinv_max = 0.;
	dtlast = dt;

	dtinv = work1;
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			if (e[i][j] < 0) {
				fprintf(stdout,
					"negative internal energy error\n");
				fprintf(stdout, "%g %d %d %10.5g %10.5g\n",
					t, i, j, S[i][j], e[i][j]);
				exit(3);
			}
			if (S[i][j] < 0) {
				fprintf(stdout,
					"negative density error\n");
				fprintf(stdout, "%g %d %d %10.5g %10.5g\n",
					t, i, j, S[i][j], e[i][j]);
				exit(4);
			}

			/* sound speed */
			if (fabs(g2 - 1.) > SMALL)
				cs = sqrt(g2 * (g2 - 1.) * e[i][j] /
					  S[i][j]);
			idt1 = (fabs(vx[i][j]) + cs) / dx;
			idt2 = (fabs(vy[i][j]) + cs) / dy;

			dtinv[i][j] = idt1 * idt1 + idt2 * idt2;

		}
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			if (dtinv[i][j] > dtinv_max) {
				dtinv_max = dtinv[i][j];
				i_crit_dt = i;
				j_crit_dt = j;
			}
		}

	dt = cour / sqrt(dtinv_max);

	/* don't increase timestep past minimum cooling time */
	if (dt > cool_fac * tcnew_min && tcnew_min != 0)
		dt = cool_fac * tcnew_min;

	/* don't increase timestep by too much */
	if (dt > 1.3 * dtlast)
		dt = 1.3 * dtlast;

	/* don't step beyond next dumping time */
	if (dt > 0.5 * DTd)
		dt = 0.5 * DTd;

	/* don't step beyond end of run */
	if (t + dt > tf)
		dt = tf - t;
}
